-- Include files to be used throughout this file.
local simquery = include( "sim/simquery" )
local simdefs = include( "sim/simdefs" )

-- Private static variables can be used by load/unload to remember what has been done.
local ThisModLoaded = false

local function earlyInit( modApi )
	-- List all mods that must load before this one, using their modinfo.txt names.
	modApi.requirements = { "Contingency Plan", "Sim Constructor", "Function Library" }
end

local function shouldUnitProvideCover( unit )
	return unit
	and not unit:getTraits().doesNotProvideCover -- special trait for mod exceptions
	and not unit:getTraits().isDrone -- drones handled by another mod
	and unit:getTraits().isAgent -- ignore mainframe devices &c
	and unit:getTraits().dynamicImpass -- just in case any guards/agents are ethereal
	and not unit:getTraits().invisible -- cloaking removes cover
	-- and not unit:isDown() -- just in case any guards/agents spawn KO/dead (handled more directly by dynamicImpass)
	and ThisModLoaded
end

local function applyUnitCover( unit )
	if shouldUnitProvideCover( unit ) then
		if not unit:getTraits().cover then
			-- log:write("covering ".. unit:getName() .." (".. unit:getID() ..")" )
			unit:getTraits().cover = true
			unit:getTraits().humansProvideCover = true
		end
	elseif unit:getTraits().humansProvideCover then
		-- log:write("uncovering ".. unit:getName() .." (".. unit:getID() ..")" )
		unit:getTraits().cover = nil
		unit:getTraits().humansProvideCover = nil
		-- cleanup after sim, which does not expect cover to disappear
		-- compare to simengine:applyImpass:64
		local cell = unit:getSim():getCell( unit:getLocation() )
		if cell then
			cell.cover = (cell.cover or 1) - 1
			for dir, exit in pairs( cell.exits ) do
				if simquery.isOpenExit( exit ) then
					for _, other in ipairs( exit.cell.units ) do
						if other:canHide() then
							other:getSim():dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = other })
						end
						other:getSim():generateSeers( other )
					end
				end
			end
		end
	end
end

-- Init runs only once, when the game boots up.
local function init( modApi )

	-- Load all kwads here.
	-- local dataPath = modApi:getDataPath()
	-- KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" )
	-- KLEIResourceMgr.MountPackage( dataPath .. "/sound.kwad", "data" )
    -- MOAIFmodDesigner.loadFEV("samplesound.fev")
	-- KLEIResourceMgr.MountPackage( dataPath .. "/anims.kwad", "data" )

	-- no mod options
	modApi.mod_manager.generationOptions[modApi.mod_id] = { name = modApi.mod_manager:getModName(modApi.mod_id), enabled = true, options = {}, icon=modApi.mod_manager:getModIcon(modApi.mod_id) }

    local scriptPath = modApi:getScriptPath()

	-- stolen and adapted from Hekateras' "Drones Provide Cover"
	local simengine = include("sim/engine")
	local spawnUnit = simengine.spawnUnit 
	simengine.spawnUnit = function( self, unit, ... )
		spawnUnit( self, unit, ... )
		applyUnitCover( unit )
	end
	
	-- update on KO, dead, cloak
	local simunit = include("sim/simunit")

	local setInvisible = simunit.setInvisible 
	simunit.setInvisible = function( self, ... )
		setInvisible( self, ... )
		applyUnitCover( self )
	end

	local setKO = simunit.setKO 
	simunit.setKO = function( self, ... )
		setKO( self, ... )
		applyUnitCover( self )
	end

	local onDamage = simunit.onDamage 
	simunit.onDamage = function( self, ... )
		onDamage( self, ... )
		applyUnitCover( self )
	end

end

-- Load runs every time a campaign gets started or resumed.
local function load( modApi, options, params, allOptions )
	ThisModLoaded = true
end

-- Unload runs every time a campaign gets quit.
local function unload( modApi, options )
	ThisModLoaded = false
end


return {
    init = init,
    earlyInit = earlyInit,
    load = load,
    unload = unload,
}
